package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.manage.enums.KfRoomMsgTypeEnum;
import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.game.module.manage.pojo.params.KfWebSocketMsgParam;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.*;
import com.zanxiang.game.module.mybatis.mapper.KfRoomMapper;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.util.function.Tuple2;
import reactor.util.function.Tuples;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2024-02-23
 * @description : 客服聊天房间
 */
@Slf4j
@Service
public class KfRoomServiceImpl extends ServiceImpl<KfRoomMapper, KfRoom> implements IKfRoomService {

    @Autowired
    private IGameAuthService gameAuthService;

    @Autowired
    private IGameAppletService gameAppletService;

    @Autowired
    private IKfRoomMsgService kfRoomMsgService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IKfSessionUserService kfSessionUserService;

    @Override
    public Tuple2<KfWebSocketMsgDTO.PageBean, List<KfWebSocketMsgDTO.RoomBean>> getFinishRoomList(Long gameId, KfWebSocketMsgParam.PageBean pageBean) {
        Page<KfRoom> kfRoomPage = super.page(new Page<>(pageBean.getPageNum(), pageBean.getPageSize()),
                new QueryWrapper<KfRoom>().lambda()
                        .eq(KfRoom::getGameId, gameId)
                        .eq(KfRoom::getOnline, Boolean.FALSE)
                        .orderByDesc(KfRoom::getUpdateTime)
        );
        List<KfWebSocketMsgDTO.RoomBean> roomBeanList = kfRoomPage.getRecords().stream()
                .map(this::transform).collect(Collectors.toList());
        return Tuples.of(this.transform(kfRoomPage), roomBeanList);
    }

    private KfWebSocketMsgDTO.PageBean transform(Page<KfRoom> kfRoomPage) {
        return KfWebSocketMsgDTO.PageBean.builder()
                .pageNum(kfRoomPage.getCurrent())
                .pageSize(kfRoomPage.getSize())
                .pageTotal(kfRoomPage.getPages())
                .total(kfRoomPage.getTotal())
                .build();
    }

    @Override
    public Long userJoinRoom(String openId, Long gameId) {
        KfRoom kfRoom = super.getOne(new LambdaQueryWrapper<KfRoom>()
                .eq(KfRoom::getOpenId, openId)
                .eq(KfRoom::getKfUserId, SecurityUtil.getUserId()));
        if (kfRoom == null) {
            kfRoom = this.transform(openId, gameId);
            super.save(kfRoom);
            return kfRoom.getId();
        }
        super.update(new LambdaUpdateWrapper<KfRoom>()
                .set(KfRoom::getOnline, Boolean.TRUE)
                .set(KfRoom::getUpdateTime, LocalDateTime.now())
                .eq(KfRoom::getId, kfRoom.getId())
        );
        return kfRoom.getId();
    }

    private KfRoom transform(String openId, Long gameId) {
        User user = userService.getOne(new LambdaQueryWrapper<User>()
                .eq(User::getGameId, gameId)
                .eq(User::getOpenId, openId));
        return KfRoom.builder()
                .gameId(gameId)
                .openId(openId)
                .userId(user == null ? null : user.getId())
                .kfUserId(SecurityUtil.getUserId())
                .online(Boolean.TRUE)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
    }

    @Override
    public List<KfWebSocketMsgDTO.GameBean> getKfGameList() {
        List<GameAuth> gameAuthList = gameAuthService.list(new LambdaQueryWrapper<GameAuth>()
                .eq(!SecurityUtil.isAdmin(), GameAuth::getUserId, SecurityUtil.getUserId()));
        if (!SecurityUtil.isAdmin() && CollectionUtils.isEmpty(gameAuthList)) {
            return Collections.emptyList();
        }
        return gameAppletService.list(new LambdaQueryWrapper<GameApplet>()
                .in(!SecurityUtil.isAdmin(), GameApplet::getGameId,
                        gameAuthList.stream().map(GameAuth::getGameId).collect(Collectors.toSet()))
                .eq(GameApplet::getType, 1)
        ).stream().map(this::transform).collect(Collectors.toList());
    }

    private KfWebSocketMsgDTO.GameBean transform(GameApplet gameApplet) {
        if (gameApplet == null) {
            return null;
        }
        int unReadMsgCount = kfRoomMsgService.count(new LambdaQueryWrapper<KfRoomMsg>()
                .eq(KfRoomMsg::getGameId, gameApplet.getGameId())
                .eq(KfRoomMsg::getReadStatus, Boolean.FALSE));
        KfWebSocketMsgDTO.GameBean gameBean = BeanUtil.copy(gameApplet, KfWebSocketMsgDTO.GameBean.class);
        gameBean.setUnReadMsgCount(unReadMsgCount);
        return gameBean;
    }

    @Override
    public List<KfWebSocketMsgDTO.RoomBean> getOnlineRoomList(Long gameId) {
        return super.list(new LambdaQueryWrapper<KfRoom>()
                .eq(KfRoom::getKfUserId, SecurityUtil.getUserId())
                .eq(KfRoom::getGameId, gameId)
                .eq(KfRoom::getOnline, Boolean.TRUE)
        ).stream().map(this::transform).collect(Collectors.toList());
    }

    private KfWebSocketMsgDTO.RoomBean transform(KfRoom kfRoom) {
        if (kfRoom == null) {
            return null;
        }
        KfWebSocketMsgDTO.RoomBean roomBean = BeanUtil.copy(kfRoom, KfWebSocketMsgDTO.RoomBean.class);
        roomBean.setRoomId(kfRoom.getId());
        //最近角色信息
        KfSessionUser kfSessionUser = kfSessionUserService.getById(kfRoom.getOpenId());
        if (kfSessionUser != null) {
            roomBean.setLastRoleId(kfSessionUser.getLastRoleId());
            roomBean.setLastRoleName(kfSessionUser.getLastRoleName());
        }
        //未读消息数量
        int unReadMsgCount = kfRoomMsgService.count(new LambdaQueryWrapper<KfRoomMsg>()
                .eq(KfRoomMsg::getRoomId, kfRoom.getId())
                .eq(KfRoomMsg::getReadStatus, Boolean.FALSE));
        roomBean.setUnReadMsgCount(unReadMsgCount);
        //最后一条消息
        KfRoomMsg kfRoomMsg = kfRoomMsgService.getOne(new LambdaQueryWrapper<KfRoomMsg>()
                .eq(KfRoomMsg::getRoomId, kfRoom.getId())
                .orderByDesc(KfRoomMsg::getCreateTime)
                .last("limit 1"));
        if (kfRoomMsg != null) {
            if (Objects.equals(kfRoomMsg.getMsgType(), KfRoomMsgTypeEnum.KF_MSG_TYPE_TEXT.getValue())) {
                KfWebSocketMsgDTO.MsgContentBean msgContent = JsonUtil.toObj(kfRoomMsg.getContent(), KfWebSocketMsgDTO.MsgContentBean.class);
                roomBean.setLastMsg(msgContent.getText());
            }
            if (Objects.equals(kfRoomMsg.getMsgType(), KfRoomMsgTypeEnum.KF_MSG_TYPE_IMAGE.getValue())) {
                roomBean.setLastMsg("[图片]");
            }
            if (Objects.equals(kfRoomMsg.getMsgType(), KfRoomMsgTypeEnum.KF_MSG_TYPE_LINK.getValue())) {
                roomBean.setLastMsg("[充值]");
            }
            roomBean.setLastMsgTime(kfRoomMsg.getCreateTime());
        }
        return roomBean;
    }
}
